@@ -11,6 +11,7 @@ from maitai._utils import __version__ as version, chat_completion_chunk_to_respo
 from maitai_common.utils.types import AsyncChunkQueue, ChunkQueue, EvaluateCallback, QueueIterable
 from maitai_gen.chat import ChatCompletionChunk, ChatCompletionParams, ChatCompletionRequest, ChatCompletionResponse, ChatStorageRequest
 from maitai_gen.inference import InferenceStreamResponse
+from maitai_gen.metric import RequestTimingMetric
 
 
 class InferenceException(Exception):
@@ -18,6 +19,11 @@ class InferenceException(Exception):
         super().__init__(msg)
 
 
+class MaitaiConnectionError(ConnectionError):
+    def __init__(self, msg: str):
+        super().__init__(msg)
+
+
 class InferenceWarning(Warning):
     def __init__(self, *args, **kwargs):
         pass
@@ -29,10 +35,7 @@ class Inference(MaitaiClient):
         super().__init__()
 
     @classmethod
-    def infer(cls, session_id, reference_id, intent, application_ref_name, completion_params: ChatCompletionParams, apply_corrections: bool, evaluation_enabled: bool, evaluate_callback: EvaluateCallback = None,
-              timeout=None, context_retrieval_enabled=False, context_query='', return_request=False, fallback_model=None, user_id: str = "") -> Iterable[InferenceStreamResponse]:
-        chat_request: ChatCompletionRequest = cls.create_inference_request(application_ref_name, session_id, reference_id, intent, apply_corrections, evaluation_enabled, completion_params, evaluate_callback,
-                                                                           context_retrieval_enabled, context_query, return_request, fallback_model, user_id)
+    def infer(cls, chat_request: ChatCompletionRequest, evaluate_callback, timeout) -> Iterable[InferenceStreamResponse]:
         if evaluate_callback:
             q = ChunkQueue()
             cls.run_async(cls.send_inference_request_async(chat_request, chunk_queue=q, evaluation_callback=evaluate_callback))
@@ -41,27 +44,24 @@ class Inference(MaitaiClient):
             return cls.send_inference_request(chat_request)
 
     @classmethod
-    async def infer_async(cls, session_id, reference_id, intent, application_ref_name, completion_params: ChatCompletionParams, apply_corrections: bool, evaluation_enabled: bool,
-                          evaluate_callback: EvaluateCallback = None, timeout=None, context_retrieval_enabled=False, context_query='', fallback_model=None, user_id="") -> AsyncIterable[InferenceStreamResponse]:
-        chat_request: ChatCompletionRequest = cls.create_inference_request(application_ref_name, session_id, reference_id, intent, apply_corrections, evaluation_enabled, completion_params, evaluate_callback,
-                                                                           context_retrieval_enabled, context_query, fallback_model, user_id)
+    async def infer_async(cls, chat_request: ChatCompletionRequest, evaluate_callback, timeout) -> AsyncIterable[InferenceStreamResponse]:
         q = AsyncChunkQueue()
         cls.run_async(cls.send_inference_request_async(chat_request, async_chunk_queue=q, evaluation_callback=evaluate_callback))
         return QueueIterable(q, timeout=timeout)
 
     @classmethod
     def create_chat_storage_request(cls, session_id, reference_id, intent, application_ref_name, completion_params: ChatCompletionParams, chat_completion_response: Optional[ChatCompletionResponse],
-                                    final_chunk: Optional[ChatCompletionChunk], content: str):
+                                    final_chunk: Optional[ChatCompletionChunk], content: str, timing: RequestTimingMetric):
         inference_request = cls.create_inference_request(application_ref_name, session_id, reference_id, intent, False, False, completion_params, None, False, '')
         if final_chunk:
             chat_completion_response = chat_completion_chunk_to_response(final_chunk, content)
 
-        return ChatStorageRequest(chat_completion_request=inference_request, chat_completion_response=chat_completion_response)
+        return ChatStorageRequest(chat_completion_request=inference_request, chat_completion_response=chat_completion_response, timing_metrics=timing)
 
     @classmethod
     def store_chat_response(cls, session_id, reference_id, intent, application_ref_name, completion_params: ChatCompletionParams, chat_completion_response: Optional[ChatCompletionResponse],
-                            final_chunk: Optional[ChatCompletionChunk], content: str):
-        chat_storage_request = cls.create_chat_storage_request(session_id, reference_id, intent, application_ref_name, completion_params, chat_completion_response, final_chunk, content)
+                            final_chunk: Optional[ChatCompletionChunk], content: str, timing: RequestTimingMetric):
+        chat_storage_request = cls.create_chat_storage_request(session_id, reference_id, intent, application_ref_name, completion_params, chat_completion_response, final_chunk, content, timing)
         cls.run_async(cls.send_storage_request_async(chat_storage_request))
 
     @classmethod
@@ -76,8 +76,7 @@ class Inference(MaitaiClient):
             }
             response = requests.post(url, headers=headers, data=chat_request.to_json(casing=Casing.SNAKE), stream=True)
             if response.status_code != 200:
-                print(f"Failed to send inference request. Status code: {response.status_code}. Error: {response.text}")
-                return
+                raise MaitaiConnectionError(f"Failed to send inference request. Status code: {response.status_code}. Error: {response.text}")
             try:
                 for line in response.iter_lines():
                     if line:
@@ -102,11 +101,11 @@ class Inference(MaitaiClient):
             async with session.post(url, headers=headers, data=chat_request.to_json(casing=Casing.SNAKE)) as response:
                 if response.status != 200:
                     error_text = await response.text()
-                    print(f"Failed to send inference request. Status code: {response.status}. Error: {error_text}")
+                    exception = MaitaiConnectionError(f"Failed to send inference request. Status code: {response.status}. Error: {error_text}")
                     if chunk_queue:
-                        chunk_queue.put(StopIteration())
+                        chunk_queue.put(exception)
                     if async_chunk_queue:
-                        await async_chunk_queue.put(StopIteration())
+                        await async_chunk_queue.put(exception)
                     return
                 async for line in response.content:
                     if line:
@@ -138,4 +137,23 @@ class Inference(MaitaiClient):
             async with aiohttp.ClientSession(connector=aiohttp.TCPConnector(ssl=False)) as session:
                 return await session.put(url, headers=headers, data=chat_storage_request.to_json(casing=Casing.SNAKE))
         except:
-            traceback.print_exc()
+            pass
+
+    @classmethod
+    def store_request_timing_data(cls, metric: RequestTimingMetric):
+        cls.run_async(cls.send_request_timing_data(metric))
+
+    @classmethod
+    async def send_request_timing_data(cls, metric: RequestTimingMetric):
+        host = config.maitai_host
+        url = f'{host}/metrics/timing'
+        headers = {
+            'Content-Type': 'application/json',
+            'x-api-key': config.api_key,
+            'x-client-version': version,
+        }
+        try:
+            async with aiohttp.ClientSession(connector=aiohttp.TCPConnector(ssl=False)) as session:
+                return await session.put(url, headers=headers, data=metric.to_json(casing=Casing.SNAKE))
+        except:
+            pass
