@@ -304,8 +304,9 @@ class Completions:
 
             )
             if stream:
-                return _process_openai_stream(session_id, reference_id, intent, application_ref_name, messages, response, evaluation_enabled, completion_params, callback)
+                return _process_openai_stream(session_id, reference_id, intent, application_ref_name, messages, response, evaluation_enabled, completion_params, timing, callback)
             else:
+                timing.time_request_end = time.time()
                 maitai_completion = convert_openai_chat_completion(response)
                 proto_messages = openai_messages_to_proto(messages)
                 proto_messages.append(
@@ -321,6 +322,7 @@ class Completions:
                         callback=callback,
                         chat_completion_response=maitai_completion,
                         completion_params=completion_params,
+                        timing=timing,
                     )
                 else:
                     maitai.Inference.store_chat_response(
@@ -331,7 +333,8 @@ class Completions:
                         chat_completion_response=maitai_completion,
                         completion_params=completion_params,
                         final_chunk=None,
-                        content=""
+                        content="",
+                        timing=timing,
                     )
                 return MaitaiCompletion(maitai_completion)
 
@@ -344,11 +347,15 @@ def _process_openai_stream(session_id: Union[str, int],
                            stream: openai.Stream[openai_chat_types.ChatCompletionChunk],
                            evaluation_enabled: bool,
                            chat_completion_params: ChatCompletionParams,
-                           callback: Optional[EvaluateCallback] = None, ) -> Iterable[MaitaiChunk]:
+                           timing: RequestTimingMetric,
+                           callback: Optional[EvaluateCallback] = None,
+                           ) -> Iterable[MaitaiChunk]:
     full_body = ""
     proto_messages = openai_messages_to_proto(messages)
     last_chunk = None
     for chunk in stream:
+        if last_chunk is None:
+            timing.time_first_chunk = time.time()
         maitai_chunk = convert_open_ai_chat_completion_chunk(chunk)
         if maitai_chunk.choices:
             last_chunk = maitai_chunk
@@ -358,6 +365,7 @@ def _process_openai_stream(session_id: Union[str, int],
         if maitai_chunk.usage and last_chunk is not None and not last_chunk.usage:
             last_chunk.usage = maitai_chunk.usage
         yield MaitaiChunk(maitai_chunk)
+    timing.time_request_end = time.time()
     if evaluation_enabled:
         proto_messages.append(ChatMessage(role="assistant", content=full_body))
         Evaluator.evaluate(
@@ -370,6 +378,7 @@ def _process_openai_stream(session_id: Union[str, int],
             callback=callback,
             chat_completion_chunk=last_chunk,
             completion_params=chat_completion_params,
+            timing=timing,
         )
     else:
         maitai.Inference.store_chat_response(
@@ -381,6 +390,7 @@ def _process_openai_stream(session_id: Union[str, int],
             final_chunk=last_chunk,
             content=full_body,
             chat_completion_response=None,
+            timing=timing,
         )
 
 
